@@ -1,9 +1,11 @@
 import sys
+from .context import spawn, BatchAsyncResult
+from .utils import transform
 
 class Scheduler(object):
     __slots__ = []
 
-    def run_pending_batch(self, id_, function, args_tuple):
+    def run_pending_batch(self, id_, function, args_tuple, **kwargs):
         raise NotImplementedError()
 
     def run_next(self):
@@ -33,7 +35,7 @@ class AllAtOnceScheduler(Scheduler):
     def __init__(self):
         self.pending_batches = {}  # {id: (function, [(args, kwargs), ...], [result, result, ...])}
 
-    def run_pending_batch(self, id_, function, args_tuple):
+    def run_pending_batch(self, id_, function, args_tuple, max_size=sys.maxint):
         if id_ not in self.pending_batches:
             aresult = BatchAsyncResult()
             arg_list = [args_tuple]
@@ -43,15 +45,23 @@ class AllAtOnceScheduler(Scheduler):
             arg_list.append(args_tuple)
 
         index = len(arg_list) - 1
-        r = aresult.get()[index]
-        if isinstance(r, Raise):
-            if len(r.exc_info) == 3:
-                exc, v, tb = r.exc_info
-                raise exc, v, tb
+
+        if index >= max_size - 1:
+            self.pending_batches.pop(id_)
+            spawn(self.run_batch_fn, function, arg_list, aresult)
+
+        def result_transform(result):
+            r = result[index]
+            if isinstance(r, Raise):
+                if len(r.exc_info) == 3:
+                    exc, v, tb = r.exc_info
+                    raise exc, v, tb
+                else:
+                    raise r.exc_info[0]
             else:
-                raise r.exc_info[0]
-        else:
-            return r
+                return r
+
+        return transform(aresult, result_transform)
 
     def has_work(self):
         return bool(self.pending_batches)
@@ -70,6 +80,3 @@ class Raise(object):
     This may be useful if you want to throw only for some of the batched cases."""
     def __init__(self, *exc_info):
         self.exc_info = exc_info
-
-# TODO: Fix this.
-from .context import spawn, BatchAsyncResult
